#vider la mmoire
rm(list=ls())

#donnes iris
data(iris)
print(str(iris))

#sous data.frame avec les 2 dernires variables
subIris <- iris[,c("Petal.Width","Species")]

#fonction pour courbe de densit conditionnelle
mydensityplot <- function(x,y,var_name=""){
  #tendue des valeurs
  etendue <- range(x)
  #test si y est factor, y rmdier sinon
  if (!is.factor(y)){
    y <- factor(y)
  }
  #calcul des courbes de densit conditionnellement  y
  d <- lapply(levels(y),function(k){density(x[y==k])})
  #ordonnee max.
  max_height <- sapply(d, function(v){return(max(v$y))})
  #palette de couleurs
  couleurs <- rainbow(nlevels(y))
  #graphique de la premire courbe
  plot(d[[1]],col=couleurs[1],ylim=c(0,max(max_height)),xlim=etendue,main=var_name)
  #les autres courbes dans le mme graphique
  for (k in 2:length(d)){
    lines(d[[k]],col=couleurs[k])
  }
}

#pour la variable  traiter
mydensityplot(subIris$Petal.Width,subIris$Species,"Petal.Width")

#chargement du package discretization
library(discretization)

#discrtisation supervise de la variable Petal.Width
discPW <- mdlp(subIris)

#bornes de discrtisation
print(discPW$cutp)

#donnes recodes
print(head(discPW$Disc.data))

#qualit du dcoupage
print(table(subIris$Species,discPW$Disc.data$Petal.Width))

#appliquer un abre de dcision sur ce dataset
library(rpart)

#paramtres d'analyse
paramArbre <- rpart.control(minsplit=20,minbucket=5,maxdepth=3,cp=0.15,xval=0,maxcompete=0,maxsurrogate=0,usesurrogate=0)

#arbre
arbre <- rpart(Species ~ ., data = subIris, control = paramArbre)
print(arbre)

#affichage sympathique
library(rpart.plot)
rpart.plot(arbre)

#description des dcoupages
print(arbre$splits)

#dlimitation des intervalles de discrtisation
bornes <- arbre$splits[,'index']
bornes <- c(-Inf,sort(bornes),+Inf)
print(bornes)

#appliquer aux donnes
discPWArbre <- cut(subIris$Petal.Width,bornes,labels=FALSE)
print(discPWArbre)

#comparaison avec celui de MDLP - discretization
print(table(discPWArbre,discPW$Disc.data$Petal.Width))

#fonction ralisant la discrtisation
mydiscVar <- function(x,y,minsplit=20,minbucket=5,maxdepth=3,cp=0.15){
  #paramtres de l'arbre
  paramTree <- rpart.control(minsplit=minsplit,minbucket=minbucket,maxdepth=maxdepth,cp=cp,xval=0,maxcompete=0,maxsurrogate=0,usesurrogate=0)
  #trier pour plus de rapidit
  index <- order(x)
  yTemp <- y[index]
  xTemp <- x[index]
  #induction de l'abre
  tree <- rpart(yTemp ~ xTemp, control=paramTree)
  #rcupration des bornes
  #si l'arbre s'est arrt  la racine
  if (is.null(tree$splits)){
    seuils <- c(-Inf,+Inf)
  } else
  {
    seuils <- c(-Inf,sort(tree$splits[,'index']),+Inf)
  }
  #appliquer aux donnes
  z <- cut(x,seuils,labels=FALSE)
  #return
  return(list(cuts=seuils,data=z))
}

#tester la fonction
discPWMine <- mydiscVar(subIris$Petal.Width,subIris$Species)

#bornes
print(discPWMine$cuts)

#variable recode
print(discPWMine$data)

#traitement de l'ensemble des variables
#densits conditionnelles
par(mfrow=c(2,2))
for (j in 1:4){
  mydensityplot(iris[,j],iris$Species,colnames(iris)[j])
}
par(mfrow=c(1,1))

#discrtisation avec mdlp
#par hypothse, var. classe en dernire position
discIrisMdlp <- mdlp(iris)

#bornes
print(discIrisMdlp$cutp)

#var. recodes
print(head(discIrisMdlp$Disc.data))

#fonction pour discrtisation data.frame
mydiscDataFrame.fit <- function(df,minsplit=20,minbucket=5,maxdepth=3,cp=0.15){
  #nombre de variables  discrtiser
  p <- ncol(df) - 1
  #discrtiser chaque variable
  cutp <- list()
  disc.data <- list()
  for (j in 1:p){
    #traiter une variable
    res <- mydiscVar(df[,j],df[,p+1],minsplit,minbucket,maxdepth,cp)
    #ajouter les rsultats
    cutp[[j]] <- res$cuts
    disc.data[[j]] <- res$data
  }
  #ajouter la dernire colonne
  disc.data[[j+1]] <- df[,p+1]
  #transformer en data.frame
  disc.data <- data.frame(disc.data)
  #renommer
  colnames(disc.data) <- colnames(df)
  #return
  return(list(cutp = cutp, disc.data = disc.data))
}

#traiter iris
discIrisArbre <- mydiscDataFrame.fit(iris)

#bornes
print(discIrisArbre$cutp)

#donnes recodes
print(head(discIrisArbre$disc.data))

#traiter le fichier wave
setwd("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/discretization/package R - discretization")
wave <- read.csv("wave100k.txt",header=T,sep="\t")
print(str(wave))

#distributions conditionnelles
par(mfrow=c(7,3))
for (j in 1:21){
  mydensityplot(wave[,j],wave$onde,colnames(wave)[j])
}
par(mfrow=c(1,1))

#mdlp
print(system.time(waveMdlp <- mdlp(wave)))

#MDL de RWeka
library(RWeka)
print(system.time(waveMdlpWeka <- Discretize(onde ~ ., data = wave)))

#
lapply(waveMdlpWeka,table)

#bornes
print(waveMdlp$cutp)

#notre algo
print(system.time(waveMine <- mydiscDataFrame.fit(wave)))

#bornes
print(waveMine$cutp)

#fichier de dploiement
waveBis <- read.csv("wave50k_deploiement.txt",header=T,sep="\t")
print(str(waveBis))

#fonction pour le dploiement
mydiscDataFrame.transform <- function(objetDisc,newdata){
  #nombre de variables  traiter
  p <- length(objetDisc$cutp)
  #vrifier les noms des variables
  ok <- (sum(colnames(objetDisc$disc.data)[1:p] == colnames(newdata)[1:p]) == p)
  #si pas ok on arrte
  if (!ok){
    stop("Variables non coherentes")
  }
  #on continue sinon
  resdata <- lapply(1:p,function(j){return(cut(newdata[,j],objetDisc$cutp[[j]],labels=FALSE))})
  #transformer en data.frame
  resdata <- data.frame(resdata)
  #renommer les variables
  colnames(resdata) <- colnames(newdata)[1:p]
  #et return
  return(list(codec=objetDisc$cutp,disc.data=resdata))
}

#encoder waveBis
print(system.time(waveBisDisc <- mydiscDataFrame.transform(objetDisc=waveMine,newdata=waveBis)))

#vrif.
print(waveBisDisc$codec)

#variables discrtises
print(head(waveBisDisc$disc.data))

#ou encore
print(summary(waveBisDisc$disc.data))
